#!/bin/sh

export LANG=C.UTF-8

PROG="./a.out"

ASM='asm volatile\(".*" : : ("s"\(\(unsigned\)blockDim\.[xyz]\)(, )?)+\);'

if ! grep -Eq "$ASM" test.hip; then
    exit 1;
fi

sed -E "s/$ASM//" test.hip > test.noasm.hip

if ! /opt/rocm/llvm/bin/clang++ -O3 -include /opt/rocm/include/hip/hip_runtime.h --cuda-device-only --offload-arch=gfx1030 test.hip -o test.hipfb \
    -Werror=uninitialized -Werror=implicit-int -Werror=integer-overflow -Werror=return-type -Werror=incompatible-pointer-types -Werror=int-conversion -Werror=pedantic -pedantic -pedantic-errors; then
    exit 1
fi

if ! /opt/rocm/llvm/bin/clang++ -O3 -include /opt/rocm/include/hip/hip_runtime.h --cuda-device-only --offload-arch=gfx1030 test.noasm.hip -o test.noasm.hipfb \
    -Werror=uninitialized -Werror=implicit-int -Werror=integer-overflow -Werror=return-type -Werror=incompatible-pointer-types -Werror=int-conversion -Werror=pedantic -pedantic -pedantic-errors; then
    exit 1
fi

for i in $(seq 1 3); do
    WITH_ASM="$(timeout 2 "$PROG" test.hipfb)"
    EXIT="$?"
    if [ "$EXIT" -ne 0 ]; then
        exit "$EXIT"
    fi

    WITHOUT_ASM="$(timeout 2 "$PROG" test.noasm.hipfb)"
    EXIT="$?"
    if [ "$EXIT" -ne 0 ]; then
        exit "$EXIT"
    fi

    DIFF="$(( ($WITHOUT_ASM - $WITH_ASM) * 100 / $WITHOUT_ASM ))"

    echo "With asm: " "$WITH_ASM" "Without: " "$WITHOUT_ASM" "$DIFF%"
    if [ "$DIFF" -lt 5 ]; then
        exit 1
    fi
done
